window.Gleam.initSearch([{"doc":"quaternion","title":"quaternion","content":"# QUATERNION\n\nA simple quaternion implementation made with and for *Gleam*. Heavily inspired by the *Rust* [quaternion](https://github.com/PistonDevelopers/quaternion) crate.\n\n## Installation\n\n```bash\ngleam add quaternion\n```\n\n## Documentation\n\nThe online documentation is available [here](https://hexdocs.pm/quaternion/).\n\n## Examples\n\n#### Construct a quaternion:\n\n```gleam\nimport gleam/io\nimport gleam_community/maths/elementary\nimport quaternion.{type Quaternion, Quaternion}\n\npub fn main() {\n  // from the constructor\n  let q = Quaternion(1.0, #(2.0, 3.0, 4.0))\n\n  // from a list\n  let r = quaternion.from_list([0.5, -2.0, 3.0, 4.0])\n  let l = quaternion.to_list(r)\n\n  // from euler angles\n  let s = quaternion.euler_angles(0.0, elementary.pi(), 0.0)\n\n  // from a rotation around an axis \n  let axis = #(1.0, 1.0, 1.0) \n  let angle = elementary.pi()\n  let t = quaternion.axis_angle(axis, angle)\n\n  io.println(quaternion.to_string(q))\n  // &quot;1.0 + 2.0i + 3.0j + 4.0k&quot;\n}\n```\n\n### Algebra on quaternions\n\n```gleam\n  // [...]\n\n  let q = Quaternion(1.0, #(2.0, 3.0, 4.0))\n  let r = quaternion.from_list([0.5, -2.0, 3.0, 4.0])\n\n  let sum = quaternion.add(q, r)\n  let diff = quaternion.substract(q, r)\n  let mult = quaternion.multiply(q, r)\n  let scale = quaternion.scale(q, 3.0)\n\n  let squared_length = quaternion.square_length(q)\n  let length = quaternion.length(q)\n  let norm = quaternion.normalize(q)\n\n  let component_mult = quaternion.times(q, r)\n  let dot_prod = quaternion.dot(q, r)\n\n// [...]\n```\n\n### Rotation\n\n```gleam\n  // [...]\n\n  let q = quaternion.euler_angles(0.0, elementary.pi(), 0.0)\n  let v = #(1.0, 2.0, 3.0)\n\n  let rot_v = quaternion.rotate_vector(q, v)\n\n  // [...]\n```\n\n### Comparison\n\n```gleam\n  // [...]\n  let epsilon = 0.0000001\n\n  let q = Quaternion(1.0, #(2.0, 3.0, 4.0))\n  let r = Quaternion(1.1, #(-2.2, 3.3, -4.4))\n  let res = Quaternion(2.1, #(-0.2, 6.3, -0.4))\n  q\n  |&gt; quaternion.add(r)\n  |&gt; quaternion.loosely_equals(res, epsilon)\n  |&gt; should.be_true\n\n  // [...]\n```\n\n## License\n\nLicensed under either of:\n\nApache License, Version 2.0, (LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0)\nMIT license (LICENSE-MIT or http://opensource.org/licenses/MIT)\nat your option.\n\n## Contribution\n\nUnless you explicitly state otherwise, any contribution intentionally submitted for inclusion in the work by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions.\n","url":"index.html"},{"doc":"vec3","title":"Vector3","content":"pub type Vector3 =\n  #(Float, Float, Float)\n The Vector3 alias.\n\nSynonyms:\nvec3.Vector3\nvec3 Vector3","url":"vec3.html#Vector3"},{"doc":"vec3","title":"zero","content":"pub const zero: #(Float, Float, Float)\n\nSynonyms:\nvec3.zero\nvec3 zero","url":"vec3.html#zero"},{"doc":"vec3","title":"add","content":"pub fn add(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Adds two vectors.\nSynonyms:\nvec3.add\nvec3 add","url":"vec3.html#add"},{"doc":"vec3","title":"cross","content":"pub fn cross(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Computes the cross product of two vectors.\nSynonyms:\nvec3.cross\nvec3 cross","url":"vec3.html#cross"},{"doc":"vec3","title":"dot","content":"pub fn dot(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n) -&gt; Float\n Computes the dot product of two vectors.\nSynonyms:\nvec3.dot\nvec3 dot","url":"vec3.html#dot"},{"doc":"vec3","title":"length","content":"pub fn length(v: #(Float, Float, Float)) -&gt; Float\n Computes the length of a vector.\nSynonyms:\nvec3.length\nvec3 length","url":"vec3.html#length"},{"doc":"vec3","title":"loosely_equals","content":"pub fn loosely_equals(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n  epsilon: Float,\n) -&gt; Bool\n Asserts if vector3 are loosely equals\nSynonyms:\nvec3.loosely_equals\nvec3 loosely_equals","url":"vec3.html#loosely_equals"},{"doc":"vec3","title":"negate","content":"pub fn negate(\n  v: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Negates a vector.\nSynonyms:\nvec3.negate\nvec3 negate","url":"vec3.html#negate"},{"doc":"vec3","title":"normalize","content":"pub fn normalize(\n  v: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Normalizes a vector.\nSynonyms:\nvec3.normalize\nvec3 normalize","url":"vec3.html#normalize"},{"doc":"vec3","title":"orthogonal","content":"pub fn orthogonal(\n  v: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Find one of the orthogonal vectors to the given vector.\nSynonyms:\nvec3.orthogonal\nvec3 orthogonal","url":"vec3.html#orthogonal"},{"doc":"vec3","title":"scale","content":"pub fn scale(\n  v: #(Float, Float, Float),\n  factor: Float,\n) -&gt; #(Float, Float, Float)\n Scales a vector by a factor.\nSynonyms:\nvec3.scale\nvec3 scale","url":"vec3.html#scale"},{"doc":"vec3","title":"square_length","content":"pub fn square_length(v: #(Float, Float, Float)) -&gt; Float\n Computes the squared length of a vector.\nSynonyms:\nvec3.square_length\nvec3 square_length","url":"vec3.html#square_length"},{"doc":"vec3","title":"substract","content":"pub fn substract(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Substracts two vectors.\nSynonyms:\nvec3.substract\nvec3 substract","url":"vec3.html#substract"},{"doc":"vec3","title":"times","content":"pub fn times(\n  v: #(Float, Float, Float),\n  w: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Multiplies (element-wise) two vectors.\nSynonyms:\nvec3.times\nvec3 times","url":"vec3.html#times"},{"doc":"vec3","title":"vec3","content":"","url":"vec3.html"},{"doc":"quaternion","title":"Quaternion","content":"pub type Quaternion {\n  Quaternion(real: Float, imaginary: #(Float, Float, Float))\n}\n The Quaternion type.\n \n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n\n     let q = Quaternion(1.0, #(2.0, 3.0, 4.0))\n \n &lt;/details&gt;\nQuaternion(real: Float, imaginary: #(Float, Float, Float))\n\n\nSynonyms:\nquaternion.Quaternion\nquaternion Quaternion","url":"quaternion.html#Quaternion"},{"doc":"quaternion","title":"id","content":"pub const id: Quaternion\n The multiplicative identity.\n \n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n\n     Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.multiply(quaternion.id)\n     |&gt; quaternion.to_string\n     // &quot;1.0 + 2.0i + 3.0j + 4.0k&quot;\n \n &lt;/details&gt; \nSynonyms:\nquaternion.id\nquaternion id","url":"quaternion.html#id"},{"doc":"quaternion","title":"add","content":"pub fn add(a: Quaternion, b: Quaternion) -&gt; Quaternion\n Adds two quaternions\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     let r = quaternion.Quaternion(0.0, #(0.0, -1.0, -2.0))\n \n     let s = quaternion.add(q, r)\n     |&gt; quaternion.to_string\n     // &quot;1.0 + 2.0i + 2.0j + 2.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.add\nquaternion add","url":"quaternion.html#add"},{"doc":"quaternion","title":"axis_angle","content":"pub fn axis_angle(\n  axis: #(Float, Float, Float),\n  angle: Float,\n) -&gt; Quaternion\n Constructs the quaternion representing the given angle (in radians) around the given axis\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     #(1.0, 1.0, 1.0)\n     |&gt; quaternion.axis_angle(elementary.pi())\n     // #(0.577, 0.577, 0.577)\n \n &lt;/details&gt;\nSynonyms:\nquaternion.axis_angle\nquaternion axis_angle","url":"quaternion.html#axis_angle"},{"doc":"quaternion","title":"conjugate","content":"pub fn conjugate(q: Quaternion) -&gt; Quaternion\n Computes the conjugate of a quaternion.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.conjugate\n     |&gt; quaternion.to_string\n     // &quot;1.0 + -2.0i + -3.0j + -4.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.conjugate\nquaternion conjugate","url":"quaternion.html#conjugate"},{"doc":"quaternion","title":"dot","content":"pub fn dot(q: Quaternion, r: Quaternion) -&gt; Float\n Computes the dot product of two quaternions.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, -2.0, -1.0))\n     let r = quaternion.Quaternion(-1.0, #(1.0, 2.0, -2.0))\n \n     let s = quaternion.dot(q, r)\n     // -1.0\n \n &lt;/details&gt;\nSynonyms:\nquaternion.dot\nquaternion dot","url":"quaternion.html#dot"},{"doc":"quaternion","title":"euler_angles","content":"pub fn euler_angles(x: Float, y: Float, z: Float) -&gt; Quaternion\n Constructs the quaternion representing the given euler angle rotations (in radian)\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.euler_angles(0.0, pi, 0.0)\n     |&gt; quaternion.rotate_vector(#(1.0, 1.0, 1.0))\n     // #(-1.0, 1.0, -1.0)\n \n &lt;/details&gt;\nSynonyms:\nquaternion.euler_angles\nquaternion euler_angles","url":"quaternion.html#euler_angles"},{"doc":"quaternion","title":"from_imaginary","content":"pub fn from_imaginary(i: #(Float, Float, Float)) -&gt; Quaternion\n Constructs a quaternion with only an imaginary part\n \n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     #(1.0, 2.0, 3.0)\n     |&gt; quaternion.from_imaginary\n     |&gt; to_string\n     // &quot;0.0 + 1.0i + 2.0j + 3.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.from_imaginary\nquaternion from_imaginary","url":"quaternion.html#from_imaginary"},{"doc":"quaternion","title":"from_list","content":"pub fn from_list(q: List(Float)) -&gt; Result(Quaternion, String)\n Constructs a Quaternion from a list of Floats.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n \n     // Is the same as:\n \n     let r = quaternion.from_list([1.0, 2.0, 3.0, 4.0])\n \n &lt;/details&gt;\nSynonyms:\nquaternion.from_list\nquaternion from_list","url":"quaternion.html#from_list"},{"doc":"quaternion","title":"from_real","content":"pub fn from_real(r: Float) -&gt; Quaternion\n Constructs a quaternion with only a real part\n \n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     1.0\n     |&gt; quaternion.from_real\n     |&gt; quaternion.to_string\n     // &quot;1.0 + 0.0i + 0.0j + 0.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.from_real\nquaternion from_real","url":"quaternion.html#from_real"},{"doc":"quaternion","title":"length","content":"pub fn length(q: Quaternion) -&gt; Float\n Computes the length of a quaternion.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.square_length\n     // 5.477...\n \n &lt;/details&gt;\nSynonyms:\nquaternion.length\nquaternion length","url":"quaternion.html#length"},{"doc":"quaternion","title":"loosely_equals","content":"pub fn loosely_equals(\n  q: Quaternion,\n  r: Quaternion,\n  epsilon: Float,\n) -&gt; Bool\n Asserts if quaternions are loosely equals\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     #(1.0, 1.0, 1.0)\n     |&gt; quaternion.axis_angle(elementary.pi())\n     |&gt; quaternion.loosely_equals(Quaternion(0.0, #(0.577, 0.577, 0.577)), epsilon)\n     |&gt; should.equal(True)\n \n &lt;/details&gt;\nSynonyms:\nquaternion.loosely_equals\nquaternion loosely_equals","url":"quaternion.html#loosely_equals"},{"doc":"quaternion","title":"multiply","content":"pub fn multiply(q: Quaternion, r: Quaternion) -&gt; Quaternion\n Computes the product of two quaternions.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     let r = quaternion.Quaternion(1.0, #(-1.0, 2.0, -2.0))\n \n     let s = quaternion.multiply(q, r)\n     |&gt; quaternion.to_string\n     // &quot;5.0 + -13.0i + 5.0j + 9.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.multiply\nquaternion multiply","url":"quaternion.html#multiply"},{"doc":"quaternion","title":"normalize","content":"pub fn normalize(q: Quaternion) -&gt; Quaternion\n Normalizes a quaternion.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.normalize\n     |&gt; quaternion.length\n     // 1.0\n \n &lt;/details&gt;\nSynonyms:\nquaternion.normalize\nquaternion normalize","url":"quaternion.html#normalize"},{"doc":"quaternion","title":"rotate_vector","content":"pub fn rotate_vector(\n  q: Quaternion,\n  v: #(Float, Float, Float),\n) -&gt; #(Float, Float, Float)\n Rotates a 3D vector using a quaternion (quaternion should be of norm 1.0).\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let v = #(0.0, 1.0, 0.0))\n     |&gt; quaternion.axis_angle(pi)\n     |&gt; quaternion.rotate_vector(#(1.0, 1.0, 1.0))\n     |&gt; quaternion.to_string\n     // &quot;0.0 + -1.0i + 1.0j + -1.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.rotate_vector\nquaternion rotate_vector","url":"quaternion.html#rotate_vector"},{"doc":"quaternion","title":"scale","content":"pub fn scale(q: Quaternion, factor: Float) -&gt; Quaternion\n Scales a quaternion (element-wise) by a scalar.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     let factor = 2.0\n \n     let s = quaternion.times(q, r)\n     |&gt; quaternion.to_string\n     // &quot;2.0 + 4.0i + 6.0j + 8.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.scale\nquaternion scale","url":"quaternion.html#scale"},{"doc":"quaternion","title":"square_length","content":"pub fn square_length(q: Quaternion) -&gt; Float\n Computes the squared length of a quaternion.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.square_length\n     // 30\n \n &lt;/details&gt;\nSynonyms:\nquaternion.square_length\nquaternion square_length","url":"quaternion.html#square_length"},{"doc":"quaternion","title":"substract","content":"pub fn substract(a: Quaternion, b: Quaternion) -&gt; Quaternion\n Substracts two quaternions\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     let r = quaternion.Quaternion(0.0, #(0.0, -1.0, -2.0))\n \n     let s = quaternion.substract(q, r)\n     |&gt; quaternion.to_string\n     // &quot;1.0 + 2.0i + 4.0j + 6.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.substract\nquaternion substract","url":"quaternion.html#substract"},{"doc":"quaternion","title":"times","content":"pub fn times(a: Quaternion, b: Quaternion) -&gt; Quaternion\n Computes the product (element-wise) of two quaternions.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     let r = quaternion.Quaternion(0.0, #(0.0, -1.0, -2.0))\n \n     let s = quaternion.times(q, r)\n     |&gt; quaternion.to_string\n     // &quot;0.0 + 0.0i + -3.0j + -4.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.times\nquaternion times","url":"quaternion.html#times"},{"doc":"quaternion","title":"to_list","content":"pub fn to_list(q: Quaternion) -&gt; List(Float)\n Constructs a list from a given Quaternion.\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.to_list\n \n     // Is the same as:\n \n     let r = [1.0, 2.0, 3.0, 4.0]\n \n &lt;/details&gt;\nSynonyms:\nquaternion.to_list\nquaternion to_list","url":"quaternion.html#to_list"},{"doc":"quaternion","title":"to_string","content":"pub fn to_string(q: Quaternion) -&gt; String\n Returns a string representing a given quaternion\n\n &lt;details&gt;\n     &lt;summary&gt;Example:&lt;/summary&gt;\n \n     let q = quaternion.Quaternion(1.0, #(2.0, 3.0, 4.0))\n     |&gt; quaternion.to_string\n \n     // Is the same as:\n     \n     let r = &quot;1.0 + 2.0i + 3.0j + 4.0k&quot;\n \n &lt;/details&gt;\nSynonyms:\nquaternion.to_string\nquaternion to_string","url":"quaternion.html#to_string"},{"doc":"quaternion","title":"quaternion","content":"","url":"quaternion.html"}]);